﻿using Castle.DynamicProxy;
using DotNetCore.CAP;
using Hotline.Share.Mq;
using Hotline.Share.Tools;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using XF.Domain.Dependency;

namespace Hotline.Settings.SystemLogDomain;
public class AsyncLoggingInterceptor : AsyncInterceptorBase
{

    private readonly ICapPublisher _capPublisher;

    public AsyncLoggingInterceptor(ICapPublisher capPublisher)
    {
        _capPublisher = capPublisher;
    }

    protected override async Task InterceptAsync(IInvocation invocation, IInvocationProceedInfo proceedInfo, Func<IInvocation, IInvocationProceedInfo, Task> proceed)
    {
        var logAttribute = GetLogAttribute(invocation);
        var logEntity = GetSystemLog(invocation, logAttribute);
        try
        {
            // 执行原始方法
            await proceed(invocation, proceedInfo);
            PublishLog(logEntity, null);
        }
        catch (Exception ex)
        {
            throw;
        }
    }

    protected override async Task<TResult> InterceptAsync<TResult>(IInvocation invocation, IInvocationProceedInfo proceedInfo, Func<IInvocation, IInvocationProceedInfo, Task<TResult>> proceed)
    {
        var logEntity = GetSystemLog(invocation, GetLogAttribute(invocation));
        try
        {
            // 执行原始方法
            TResult result = await proceed(invocation, proceedInfo);
            PublishLog(logEntity, result);
            return result;
        }
        catch (Exception ex)
        {
            throw;
        }
    }

    private LogFilterAttribute? GetLogAttribute(IInvocation invocation)
    {
        var methodInfo = invocation.MethodInvocationTarget ?? invocation.Method;
        return methodInfo.GetCustomAttributes(typeof(LogFilterAttribute), true)
                                     .FirstOrDefault() as LogFilterAttribute;
    }

    private SystemLog GetSystemLog(IInvocation invocation, LogFilterAttribute logAttribute)
    {
        if (logAttribute == null) return null;
        var entity = new SystemLog
        {
            Name = invocation.Method.Name,
        };
        for (int i = 0;i < invocation.Arguments.Length;i++)
        {
            var argument = invocation.Arguments[i];
            if (argument == null || argument is CancellationToken) continue;
            entity.ExecuteParam = argument;
            Type argType = argument.GetType();

            var properties = argType.GetProperties(BindingFlags.Public | BindingFlags.Instance);
            foreach (var property in properties)
            {
                var value = property.GetValue(argument);
                if (value != null && property.Name.ToUpper() == logAttribute.ExternalIdName.ToUpper())
                {
                    entity.ExternalId = value.ToString();
                    break;
                }
            }
        }
        return entity;
    }

    private void PublishLog(SystemLog logEntity, object? result)
    {
        if (logEntity != null)
        {
            logEntity.ExecuteResult = result;
            _capPublisher.Publish(EventNames.LoggingInterceptor, logEntity);
        }
    }
}

public class LoggingInterceptorHandler : ICapSubscribe, ITransientDependency
{
    private readonly ISystemLogRepository _systemLogRepository;

    public LoggingInterceptorHandler(ISystemLogRepository systemLogRepository)
    {
        _systemLogRepository = systemLogRepository;
    }

    [CapSubscribe(EventNames.LoggingInterceptor)]
    public async Task Handle(SystemLog logEntity)
    {
        await _systemLogRepository.AddAsync(logEntity);
    }
}
